package vm;

public class Generate {

    public static java.util.LinkedList GenVMCode (java.util.LinkedList treeCode) {
       java.util.ListIterator j = treeCode.listIterator();
       while (j.hasNext()) {
           tree.Tree t = (tree.Tree) j.next();
           generate(t);
       }
       return null;
    }

    public static void generate(tree.Tree t) {
      String nodeName = t.getClass().getName();
      int position = t.position;
      System.out.println("vm.Generate: " + nodeName + " @ " + position);
      if (t instanceof tree.Null) {
         tree.Null s = (tree.Null)t;
      } else if (t instanceof tree.Actual) {
         tree.Actual s = (tree.Actual)t;
         sym.TypeSy type = s.type;
         sym.FormalSy formal = s.formal;
         generate(s.expr);
         generate(s.next);
      } else if (t instanceof tree.AddressOf) {
         tree.AddressOf s = (tree.AddressOf)t;
         sym.VariableSy symbol = s.symbol;
         sym.TypeSy type = s.type;
      } else if (t instanceof tree.BinExpr) {
         tree.BinExpr s = (tree.BinExpr)t;
         sym.TypeSy type = s.type;
         int op = s.op;
         generate(s.left);
         generate(s.right);
      } else if (t instanceof tree.Branch) {
         tree.Branch s = (tree.Branch)t;
         int op = s.op;
         sym.TypeSy type = s.type;
         sym.Label label = s.label;
         generate(s.left);
         generate(s.right);
      } else if (t instanceof tree.Copy) {
         tree.Copy s = (tree.Copy)t;
         sym.TypeSy type = s.type;
         generate(s.toDes);
         generate(s.fromDes);
      } else if (t instanceof tree.FieldOf) {
         tree.FieldOf s = (tree.FieldOf)t;
         sym.FieldSy field = s.field;
         sym.TypeSy type = s.type;
         sym.RecordType record = s.record;
         generate(s.base);
      } else if (t instanceof tree.FormalDecl) {
         tree.FormalDecl s = (tree.FormalDecl)t;
         sym.Symbol symbol = s.symbol;
      } else if (t instanceof tree.Goto) {
         tree.Goto s = (tree.Goto)t;
         sym.Label label = s.label;
      } else if (t instanceof tree.IndexOf) {
         tree.IndexOf s = (tree.IndexOf)t;
         sym.TypeSy type = s.type;
         sym.ArrayType array = s.array;
         generate(s.base);
         generate(s.index);
      } else if (t instanceof tree.Label) {
         tree.Label s = (tree.Label)t;
         sym.Label label = s.label;
      } else if (t instanceof tree.Load) {
         tree.Load s = (tree.Load)t;
         sym.TypeSy type = s.type;
         generate(s.des);
      } else if (t instanceof tree.LoadClassTemplate) {
	  // Ignore this tree!
         tree.LoadClassTemplate s = (tree.LoadClassTemplate)t;
      } else if (t instanceof tree.LoadLit) {
         tree.LoadLit s = (tree.LoadLit)t;
         sym.Value value = s.value;
         sym.TypeSy type = s.type;
      } else if (t instanceof tree.LoadNil) {
         tree.LoadNil s = (tree.LoadNil)t;
         sym.TypeSy type = s.type;
      } else if (t instanceof tree.MethodCall) {
	  // Ignore this tree!
         tree.MethodCall s = (tree.MethodCall)t;
         sym.MethodSy symbol = s.symbol;
         sym.FormalSy self = s.self;
         generate(s.base);
         generate(s.actuals);
      } else if (t instanceof tree.ObjectFieldOf) {
	  // Ignore this tree!
         tree.ObjectFieldOf s = (tree.ObjectFieldOf)t;
         sym.FieldSy field = s.field;
         sym.ClassType class_ = s.class_;
         generate(s.base);
      } else if (t instanceof tree.NarrowVal) {
	  // Ignore this tree!
         tree.NarrowVal s = (tree.NarrowVal)t;
         tree.Expression expr = s.expr;
         tree.LoadClassTemplate template = s.template;
         generate(s.expr);
         generate(s.template);
      } else if (t instanceof tree.NarrowVar) {
	  // Ignore this tree!
         tree.NarrowVar s = (tree.NarrowVar)t;
         tree.Designator des = s.des;
         tree.LoadClassTemplate template = s.template;
         generate(s.des);
         generate(s.template);
      } else if (t instanceof tree.Special) {
	  // Ignore this tree!
         tree.Special s = (tree.Special)t;
         String value = s.value;
      } else if (t instanceof tree.ProcBegin) {
         tree.ProcBegin s = (tree.ProcBegin)t;
         sym.ProcedureSy symbol = s.symbol;
      } else if (t instanceof tree.ProcCall) {
         tree.ProcCall s = (tree.ProcCall)t;
         sym.ProcedureSy symbol = s.symbol;
         generate(s.actuals);
      } else if (t instanceof tree.ProcEnd) {
         tree.ProcEnd s = (tree.ProcEnd)t;
         sym.ProcedureSy symbol = s.symbol;
      } else if (t instanceof tree.Read) {
         tree.Read s = (tree.Read)t;
         sym.TypeSy type = s.type;
         generate(s.des);
      } else if (t instanceof tree.Store) {
         tree.Store s = (tree.Store)t;
         sym.TypeSy type = s.type;
         generate(s.left);
         generate(s.right);
      } else if (t instanceof tree.TypeDecl) {
         tree.TypeDecl s = (tree.TypeDecl)t;
         sym.Symbol symbol = s.symbol;
      } else if (t instanceof tree.UnaryExpr) {
         tree.UnaryExpr s = (tree.UnaryExpr)t;
         sym.TypeSy type = s.type;
         int op = s.op;
         generate(s.left);
      } else if (t instanceof tree.VarDecl) {
         tree.VarDecl s = (tree.VarDecl)t;
         sym.Symbol symbol = s.symbol;
      } else if (t instanceof tree.Version) {
         tree.Version s = (tree.Version)t;
         int major = s.major;
         int minor = s.minor;
      } else if (t instanceof tree.Write) {
         tree.Write s = (tree.Write)t;
         sym.TypeSy type = s.type;
         generate(s.expr);
      } else if (t instanceof tree.WriteLn) {
         tree.WriteLn s = (tree.WriteLn)t;
      }
    }
}       

      